@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2007  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1997--2013  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +24,7 @@
 # include <config.h>
 #endif
 
+#define R_USE_SIGNALS 1
 #include "Defn.h"
 #include <R_ext/Riconv.h>
 #include "Fileio.h"
@@ -35,7 +35,8 @@
 #include "getline/getline.h"
 #include "getline/wc_history.h"
 #define WIN32_LEAN_AND_MEAN 1
-#ifndef _WIN32_WINNT /* currently MinGW does not define, MinGW-w64 does */
+/* Mingw-w64 defines this to be 0x0502 */
+#ifndef _WIN32_WINNT
 #define _WIN32_WINNT 0x0500     /* for MEMORYSTATUSEX */
 #endif
 #include <windows.h>		/* for CreateEvent,.. */
@@ -50,7 +51,7 @@
 
 #include "win-nls.h"
 
-void R_CleanTempDir(void);		/* from extra.c */
+void R_CleanTempDir(void);		/* from platform.c */
 void editorcleanall(void);                  /* from editor.c */
 
 int Rwin_graphicsx = -25, Rwin_graphicsy = 0;
@@ -238,7 +239,7 @@ GuiReadConsole(const char *prompt, char *buf, int len, int addtohistory)
 {
     int res;
     const char *NormalPrompt =
-	CHAR(STRING_ELT(GetOption(install("prompt"), R_BaseEnv), 0));
+	CHAR(STRING_ELT(GetOption1(install("prompt")), 0));
 
     if(!R_is_running) {
 	R_is_running = 1;
@@ -322,8 +323,8 @@ FileReadConsole(const char *prompt, char *buf, int len, int addhistory)
     /* translate if necessary */
     if(strlen(R_StdinEnc) && strcmp(R_StdinEnc, "native.enc")) {
 	size_t res, inb = strlen(buf), onb = len;
-	const char *ib = buf; char *ob, *obuf;
-	ob = obuf = alloca(len+1);
+	const char *ib = buf; 
+	char obuf[len+1], *ob = obuf;
 	if(!cd) {
 	    cd = Riconv_open("", R_StdinEnc);
 	    if(!cd) error(_("encoding '%s' is not recognised"), R_StdinEnc);
@@ -345,8 +346,10 @@ FileReadConsole(const char *prompt, char *buf, int len, int addhistory)
 	buf[ll++] = '\n'; buf[ll] = '\0';
     }
 
-    if (!R_Interactive && !R_Slave)
+    if (!R_Interactive && !R_Slave) {
 	fputs(buf, stdout);
+	fflush(stdout);
+    }
     return 1;
 }
 
@@ -444,6 +447,8 @@ void R_CleanUp(SA_TYPE saveact, int status, int runLast)
 		saveact = SA_NOSAVE;
 		break;
 	    case CANCEL:
+		// There might be residual events with destroyed handles
+		R_ProcessEvents();
 		jump_to_toplevel();
 		break;
 
@@ -473,18 +478,19 @@ void R_CleanUp(SA_TYPE saveact, int status, int runLast)
     R_RunExitFinalizers();
     editorcleanall();
     CleanEd();
-    R_CleanTempDir();
-    KillAllDevices();
-    AllDevicesKilled = TRUE;
+    KillAllDevices(); /* Unix does not do this under SA_SUICIDE */
+    AllDevicesKilled = TRUE; /* used in devWindows.c to inhibit callbacks */
+    R_CleanTempDir(); /* changes directory */
     if (R_Interactive && CharacterMode == RTerm)
 	SetConsoleTitle(oldtitle);
     if (R_CollectWarnings && saveact != SA_SUICIDE
-	&& CharacterMode == RTerm)
-	PrintWarnings();
+	&& CharacterMode == RTerm)   /* no point in doing this for Rgui
+					as the console is about to close */
+	PrintWarnings();        /* from device close and (if run) .Last */
     app_cleanup();
     RConsole = NULL;
-    if(ifp) fclose(ifp);
-    if(ifile[0]) unlink(ifile);
+    if(ifp) fclose(ifp);        /* input file from -f or --file= */
+    if(ifile[0]) unlink(ifile); /* input file from -e */
     exit(status);
 }
 
@@ -536,6 +542,8 @@ int R_ShowFiles(int nfile, const char **file, const char **headers,
 			}
 			fclose(f);
 			if (del) DeleteFile(file[i]);
+			/* add a blank line */
+			R_WriteConsole("", 0);
 		    }
 		    else {
 			snprintf(buf, 1024,
@@ -549,7 +557,7 @@ int R_ShowFiles(int nfile, const char **file, const char **headers,
 			snprintf(buf, 1024, "\"%s\" \"%s\"", pager, file[i]);
 		    else
 			snprintf(buf, 1024, "%s \"%s\"", pager, file[i]);
-		    runcmd(buf, CE_NATIVE, 0, 1, "");
+		    runcmd(buf, CE_NATIVE, 0, 1, NULL, NULL, NULL);
 		}
 	    } else {
 		snprintf(buf, 1024,
@@ -595,7 +603,7 @@ int R_EditFiles(int nfile, const char **file, const char **title,
 		    snprintf(buf, 1024, "\"%s\" \"%s\"", editor, file[i]);
 		else
 		    snprintf(buf, 1024, "%s \"%s\"", editor, file[i]);
-		runcmd(buf, CE_UTF8, 0, 1, "");
+		runcmd(buf, CE_UTF8, 0, 1, NULL, NULL, NULL);
 	    }
 
 	}
@@ -642,7 +650,7 @@ static int char_YesNoCancel(const char *s)
     char  ss[128];
     unsigned char a[3];
 
-    sprintf(ss, "%s [y/n/c]: ", s);
+    snprintf(ss, 128, "%s [y/n/c]: ", s);
     R_ReadConsole(ss, a, 3, 0);
     switch (a[0]) {
     case 'y':
@@ -661,7 +669,7 @@ static int char_YesNoCancel(const char *s)
 
 static char RHome[MAX_PATH + 7];
 static char UserRHome[MAX_PATH + 7];
-extern char *getRHOME(void), *getRUser(void); /* in rhome.c */
+extern char *getRHOME(int), *getRUser(void); /* in rhome.c */
 void R_setStartTime(void);
 
 
@@ -689,7 +697,8 @@ void R_SetWin32(Rstart Rp)
     R_CStackDir = 1;
     R_Home = Rp->rhome;
     if(strlen(R_Home) >= MAX_PATH) R_Suicide("Invalid R_HOME");
-    sprintf(RHome, "R_HOME=%s", R_Home);
+    snprintf(RHome, MAX_PATH+7, "R_HOME=%s", R_Home);
+    for (char *p = RHome; *p; p++) if (*p == '\\') *p = '/';
     putenv(RHome);
     strcpy(UserRHome, "R_USER=");
     strcat(UserRHome, Rp->home);
@@ -774,11 +783,11 @@ char *PrintUsage(void)
 {
     static char msg[5000];
     char msg0[] =
-	"Start R, a system for statistical computation and graphics, with the\nspecified options\n\nEnvVars: Environmental variables can be set by NAME=value strings\n\nOptions:\n  -h, --help            Print usage message and exit\n  --version             Print version info and exit\n  --encoding=enc        Specify encoding to be used for stdin\n  --save                Do save workspace at the end of the session\n  --no-save             Don't save it\n",
+	"Start R, a system for statistical computation and graphics, with the\nspecified options\n\nEnvVars: Environmental variables can be set by NAME=value strings\n\nOptions:\n  -h, --help            Print usage message and exit\n  --version             Print version info and exit\n  --encoding=enc        Specify encoding to be used for stdin\n  --encoding enc        ditto\n  --save                Do save workspace at the end of the session\n  --no-save             Don't save it\n",
 	msg1[] =
 	"  --no-environ          Don't read the site and user environment files\n  --no-site-file        Don't read the site-wide Rprofile\n  --no-init-file        Don't read the .Rprofile or ~/.Rprofile files\n  --restore             Do restore previously saved objects at startup\n  --no-restore-data     Don't restore previously saved objects\n  --no-restore-history  Don't restore the R history file\n  --no-restore          Don't restore anything\n",
 	msg2[] =
-	"  --vanilla             Combine --no-save, --no-restore, --no-site-file,\n                          --no-init-file and --no-environ\n  --min-vsize=N         Set vector heap min to N bytes; '4M' = 4 MegaB\n  --max-vsize=N         Set vector heap max to N bytes;\n  --min-nsize=N         Set min number of cons cells to N\n  --max-nsize=N         Set max number of cons cells to N\n",
+	"  --vanilla             Combine --no-save, --no-restore, --no-site-file,\n                          --no-init-file and --no-environ\n",
 	msg2b[] =
 	"  --max-mem-size=N      Set limit for memory to be used by R\n  --max-ppsize=N        Set max size of protect stack to N\n",
 	msg3[] =
@@ -818,7 +827,6 @@ void R_setupHistory(void)
 }
 
 #include <sys/stat.h>
-#include <time.h>
 static int isDir(char *path)
 {
     struct stat sb;
@@ -833,6 +841,8 @@ static int isDir(char *path)
     return isdir;
 }
 
+extern void BindDomain(char *R_Home);
+
 int cmdlineoptions(int ac, char **av)
 {
     int   i, ierr;
@@ -840,28 +850,19 @@ int cmdlineoptions(int ac, char **av)
     char *p;
     char  s[1024], cmdlines[10000];
     R_size_t Virtual;
-#ifdef ENABLE_NLS
-    char localedir[PATH_MAX+20];
-#endif
     structRstart rstart;
     Rstart Rp = &rstart;
     Rboolean usedRdata = FALSE, processing = TRUE;
 
     /* ensure R_Home gets set early: we are in rgui or rterm here */
-    R_Home = getRHOME();
+    R_Home = getRHOME(3);
     /* need this for moduleCdynload for iconv.dll */
     InitFunctionHashing();
-    sprintf(RHome, "R_HOME=%s", R_Home);
+    snprintf(RHome, MAX_PATH+7, "R_HOME=%s", R_Home);
     putenv(RHome);
-#ifdef ENABLE_NLS
-    strcpy(localedir, R_Home); strcat(localedir, "/share/locale");
-    bindtextdomain("RGui", localedir);
-    bindtextdomain(PACKAGE, localedir);
-#endif
+    BindDomain(R_Home);
 
-#ifdef _R_HAVE_TIMING_
     R_setStartTime();
-#endif
 
     /* Store the command line arguments before they are processed
        by the different option handlers. We do this here so that
@@ -954,6 +955,18 @@ int cmdlineoptions(int ac, char **av)
 
     R_common_command_line(&ac, av, Rp);
 
+    char *q = getenv("R_WIN_INTERNET2");
+    if (q && q[0]) UseInternet2 = TRUE;
+    q = getenv("R_MAX_MEM_SIZE");
+    if (q && q[0]) {
+	value = R_Decode2Long(q, &ierr);
+	if(ierr || value < 32 * Mega || value > Virtual) {
+	    snprintf(s, 1024,
+		     _("WARNING: R_MAX_MEM_SIZE value is invalid: ignored\n"));
+	    R_ShowMessage(s);
+	} else R_max_memory = value;
+    }
+
     cmdlines[0] = '\0';
     while (--ac) {
 	if (processing && **++av == '-') {
@@ -987,20 +1000,24 @@ int cmdlineoptions(int ac, char **av)
 		value = R_Decode2Long(p, &ierr);
 		if(ierr) {
 		    if(ierr < 0)
-			sprintf(s, _("WARNING: --max-mem-size value is invalid: ignored\n"));
+			snprintf(s, 1024,
+				 _("WARNING: --max-mem-size value is invalid: ignored\n"));
 		    else
-			sprintf(s, _("WARNING: --max-mem-size=%lu%c: too large and ignored\n"),
+			snprintf(s, 1024,
+				 _("WARNING: --max-mem-size=%lu%c: too large and ignored\n"),
 				(unsigned long) value,
 				(ierr == 1) ? 'M': ((ierr == 2) ? 'K': 'G'));
 		    R_ShowMessage(s);
 		} else if (value < 32 * Mega) {
-		    sprintf(s, _("WARNING: --max-mem-size=%4.1fM: too small and ignored\n"),
-			    value/(1024.0 * 1024.0));
+		    snprintf(s, 1024,
+			     _("WARNING: --max-mem-size=%4.1fM: too small and ignored\n"),
+			     value/(1024.0 * 1024.0));
 		    R_ShowMessage(s);
 		} else if (value > Virtual) {
-		    sprintf(s, _("WARNING: --max-mem-size=%4.0fM: too large and taken as %uM\n"),
-			    value/(1024.0 * 1024.0),
-			    (unsigned int) (Virtual/(1024.0 * 1024.0)));
+		    snprintf(s, 1024,
+			     _("WARNING: --max-mem-size=%4.0fM: too large and taken as %uM\n"),
+			     value/(1024.0 * 1024.0),
+			     (unsigned int) (Virtual/(1024.0 * 1024.0)));
 		    R_max_memory = Virtual;
 		    R_ShowMessage(s);
 		} else
@@ -1076,9 +1093,6 @@ int cmdlineoptions(int ac, char **av)
 	if(ifp) R_Suicide(_("cannot use -e with -f or --file"));
 	Rp->R_Interactive = FALSE;
 	Rp->ReadConsole = FileReadConsole;
-	/* tmpfile() seems not to work on Vista: it tries to write in c:/
-	ifp = tmpfile();
-	*/
 	{
 	    char *tm;
 	    tm = getenv("TMPDIR");
@@ -1090,8 +1104,9 @@ int cmdlineoptions(int ac, char **av)
 			tm = getenv("R_USER"); /* this one will succeed */
 		}
 	    }
-	    srand( (unsigned) time(NULL) );
-	    sprintf(ifile, "%s/Rscript%x%x", tm, rand(), rand());
+	    /* in case getpid() is not unique -- has been seen under Windows */
+	    snprintf(ifile, 1024, "%s/Rscript%x%x", tm, getpid(), 
+		     (unsigned int) GetTickCount());
 	    ifp = fopen(ifile, "w+b");
 	    if(!ifp) R_Suicide(_("creation of tmpfile failed -- set TMPDIR suitably?"));
 	}
