@@ -1,6 +1,8 @@
 #  File src/library/stats/R/lm.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -25,7 +27,7 @@ lm <- function (formula, data, subset, weights, na.action,
     cl <- match.call()
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "weights", "na.action", "offset"),
-	       names(mf), 0L)
+               names(mf), 0L)
     mf <- mf[c(1L, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
@@ -114,39 +116,32 @@ lm.fit <- function (x, y, offset = NULL, method = "qr", tol = 1e-07,
     else if(length(dots) == 1L)
 	warning("extra argument ", sQuote(names(dots)),
                 " is disregarded.", domain = NA)
-    storage.mode(x) <- "double"
-    storage.mode(y) <- "double"
-    z <- .Fortran("dqrls",
-		  qr = x, n = n, p = p,
-		  y = y, ny = ny,
-		  tol = as.double(tol),
-		  coefficients = mat.or.vec(p, ny),
-		  residuals = y, effects = y, rank = integer(1L),
-		  pivot = 1L:p, qraux = double(p), work = double(2*p),
-                  PACKAGE="base")
+    z <- .Call(C_Cdqrls, x, y, tol)
     if(!singular.ok && z$rank < p) stop("singular fit encountered")
     coef <- z$coefficients
     pivot <- z$pivot
     ## careful here: the rank might be 0
     r1 <- seq_len(z$rank)
-    dn <- colnames(x); if(is.null(dn)) dn <- paste("x", 1L:p, sep="")
+    dn <- colnames(x); if(is.null(dn)) dn <- paste0("x", 1L:p)
     nmeffects <- c(dn[pivot[r1]], rep.int("", n - z$rank))
     r2 <- if(z$rank < p) (z$rank+1L):p else integer()
     if (is.matrix(y)) {
 	coef[r2, ] <- NA
-	coef[pivot, ] <- coef
+	if(z$pivoted) coef[pivot, ] <- coef
 	dimnames(coef) <- list(dn, colnames(y))
 	dimnames(z$effects) <- list(nmeffects, colnames(y))
     } else {
 	coef[r2] <- NA
-	coef[pivot] <- coef
+        ## avoid copy
+	if(z$pivoted) coef[pivot] <- coef
 	names(coef) <- dn
 	names(z$effects) <- nmeffects
     }
     z$coefficients <- coef
     r1 <- y - z$residuals ; if(!is.null(offset)) r1 <- r1 + offset
+    ## avoid unnecessary copy
+    if(z$pivoted) colnames(z$qr) <- colnames(x)[z$pivot]
     qr <- z[c("qr", "qraux", "pivot", "tol", "rank")]
-    colnames(qr$qr) <- colnames(x)[qr$pivot]
     c(z[c("coefficients", "residuals", "effects", "rank")],
       list(fitted.values = r1, assign = attr(x, "assign"),
 	   qr = structure(qr, class="qr"),
@@ -200,32 +195,23 @@ lm.wfit <- function (x, y, w, offset = NULL, method = "qr", tol = 1e-7,
                     fitted.values = 0 * y, weights = w, rank = 0L,
                     df.residual = length(y)))
     }
-    storage.mode(y) <- "double"
     wts <- sqrt(w)
-    z <- .Fortran("dqrls",
-		  qr = x * wts, n = n, p = p,
-		  y  = y * wts, ny = ny,
-		  tol = as.double(tol),
-		  coefficients = mat.or.vec(p, ny), residuals = y,
-		  effects = mat.or.vec(n, ny),
-		  rank = integer(1L), pivot = 1L:p, qraux = double(p),
-		  work = double(2 * p),
-                  PACKAGE="base")
+    z <- .Call(C_Cdqrls, x * wts, y * wts, tol)
     if(!singular.ok && z$rank < p) stop("singular fit encountered")
     coef <- z$coefficients
     pivot <- z$pivot
     r1 <- seq_len(z$rank)
-    dn <- colnames(x); if(is.null(dn)) dn <- paste("x", 1L:p, sep="")
+    dn <- colnames(x); if(is.null(dn)) dn <- paste0("x", 1L:p)
     nmeffects <- c(dn[pivot[r1]], rep.int("", n - z$rank))
     r2 <- if(z$rank < p) (z$rank+1L):p else integer()
     if (is.matrix(y)) {
 	coef[r2, ] <- NA
-	coef[pivot, ] <- coef
+	if(z$pivoted) coef[pivot, ] <- coef
 	dimnames(coef) <- list(dn, colnames(y))
 	dimnames(z$effects) <- list(nmeffects,colnames(y))
     } else {
 	coef[r2] <- NA
-	coef[pivot] <- coef
+	if(z$pivoted) coef[pivot] <- coef
 	names(coef) <- dn
 	names(z$effects) <- nmeffects
     }
@@ -254,8 +240,8 @@ lm.wfit <- function (x, y, w, offset = NULL, method = "qr", tol = 1e-7,
     }
     if(!is.null(offset))
         z$fitted.values <- z$fitted.values + offset
+    if(z$pivoted) colnames(z$qr) <- colnames(x)[z$pivot]
     qr <- z[c("qr", "qraux", "pivot", "tol", "rank")]
-    colnames(qr$qr) <- colnames(x)[qr$pivot]
     c(z[c("coefficients", "residuals", "fitted.values", "effects",
 	  "weights", "rank")],
       list(assign = x.asgn,
@@ -263,14 +249,14 @@ lm.wfit <- function (x, y, w, offset = NULL, method = "qr", tol = 1e-7,
 	   df.residual = n - z$rank))
 }
 
-print.lm <- function(x, digits = max(3, getOption("digits") - 3), ...)
+print.lm <- function(x, digits = max(3L, getOption("digits") - 3L), ...)
 {
     cat("\nCall:\n",
-	paste(deparse(x$call), sep="\n", collapse = "\n"), "\n\n", sep="")
+	paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
     if(length(coef(x))) {
         cat("Coefficients:\n")
-        print.default(format(coef(x), digits=digits),
-                      print.gap = 2, quote = FALSE)
+        print.default(format(coef(x), digits = digits),
+                      print.gap = 2L, quote = FALSE)
     } else cat("No coefficients\n")
     cat("\n")
     invisible(x)
@@ -292,13 +278,13 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
             r <- sqrt(w) * r
         }
         resvar <- rss/rdf
-        ans <- z[c("call", "terms")]
+        ans <- z[c("call", "terms", if(!is.null(z$weights)) "weights")]
         class(ans) <- "summary.lm"
         ans$aliased <- is.na(coef(object))  # used in print method
         ans$residuals <- r
         ans$df <- c(0L, n, length(ans$aliased))
         ans$coefficients <- matrix(NA, 0L, 4L)
-        dimnames(ans$coefficients)<-
+        dimnames(ans$coefficients) <-
             list(NULL, c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
         ans$sigma <- sqrt(resvar)
         ans$r.squared <- ans$adj.r.squared <- 0
@@ -334,11 +320,11 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
     se <- sqrt(diag(R) * resvar)
     est <- z$coefficients[Qr$pivot[p1]]
     tval <- est/se
-    ans <- z[c("call", "terms")]
+    ans <- z[c("call", "terms", if(!is.null(z$weights)) "weights")]
     ans$residuals <- r
     ans$coefficients <-
 	cbind(est, se, tval, 2*pt(abs(tval), rdf, lower.tail = FALSE))
-    dimnames(ans$coefficients)<-
+    dimnames(ans$coefficients) <-
 	list(names(z$coefficients)[Qr$pivot[p1]],
 	     c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
     ans$aliased <- is.na(coef(object))  # used in print method
@@ -364,24 +350,24 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
 }
 
 print.summary.lm <-
-    function (x, digits = max(3, getOption("digits") - 3),
+    function (x, digits = max(3L, getOption("digits") - 3L),
               symbolic.cor = x$symbolic.cor,
-	      signif.stars= getOption("show.signif.stars"),	...)
+	      signif.stars = getOption("show.signif.stars"),	...)
 {
     cat("\nCall:\n", # S has ' ' instead of '\n'
-	paste(deparse(x$call), sep="\n", collapse = "\n"), "\n\n", sep="")
+	paste(deparse(x$call), sep="\n", collapse = "\n"), "\n\n", sep = "")
     resid <- x$residuals
     df <- x$df
     rdf <- df[2L]
-    cat(if(!is.null(x$w) && diff(range(x$w))) "Weighted ",
-        "Residuals:\n", sep="")
+    cat(if(!is.null(x$weights) && diff(range(x$weights))) "Weighted ",
+        "Residuals:\n", sep = "")
     if (rdf > 5L) {
 	nam <- c("Min", "1Q", "Median", "3Q", "Max")
 	rq <- if (length(dim(resid)) == 2L)
 	    structure(apply(t(resid), 1L, quantile),
 		      dimnames = list(nam, dimnames(resid)[[2L]]))
 	else  {
-            zz <- zapsmall(quantile(resid), digits + 1)
+            zz <- zapsmall(quantile(resid), digits + 1L)
             structure(zz, names = nam)
         }
 	print(rq, digits = digits, ...)
@@ -389,7 +375,8 @@ print.summary.lm <-
     else if (rdf > 0L) {
 	print(resid, digits = digits, ...)
     } else { # rdf == 0 : perfect fit!
-	cat("ALL", df[1L], "residuals are 0: no residual degrees of freedom!\n")
+	cat("ALL", df[1L], "residuals are 0: no residual degrees of freedom!")
+        cat("\n")
     }
     if (length(x$aliased) == 0L) {
         cat("\nNo Coefficients\n")
@@ -405,21 +392,24 @@ print.summary.lm <-
             coefs[!aliased, ] <- x$coefficients
         }
 
-        printCoefmat(coefs, digits=digits, signif.stars=signif.stars, na.print="NA", ...)
+        printCoefmat(coefs, digits = digits, signif.stars = signif.stars,
+                     na.print = "NA", ...)
     }
     ##
     cat("\nResidual standard error:",
-	format(signif(x$sigma, digits)), "on", rdf, "degrees of freedom\n")
-    if(nzchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep="")
+	format(signif(x$sigma, digits)), "on", rdf, "degrees of freedom")
+    cat("\n")
+    if(nzchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep = "")
     if (!is.null(x$fstatistic)) {
-	cat("Multiple R-squared:", formatC(x$r.squared, digits=digits))
-	cat(",\tAdjusted R-squared:",formatC(x$adj.r.squared,digits=digits),
-	    "\nF-statistic:", formatC(x$fstatistic[1L], digits=digits),
+	cat("Multiple R-squared: ", formatC(x$r.squared, digits = digits))
+	cat(",\tAdjusted R-squared: ",formatC(x$adj.r.squared, digits = digits),
+	    "\nF-statistic:", formatC(x$fstatistic[1L], digits = digits),
 	    "on", x$fstatistic[2L], "and",
 	    x$fstatistic[3L], "DF,  p-value:",
 	    format.pval(pf(x$fstatistic[1L], x$fstatistic[2L],
-                           x$fstatistic[3L], lower.tail = FALSE), digits=digits),
-	    "\n")
+                           x$fstatistic[3L], lower.tail = FALSE),
+                        digits = digits))
+        cat("\n")
     }
     correl <- x$correlation
     if (!is.null(correl)) {
@@ -492,7 +482,9 @@ simulate.lm <- function(object, nsim = 1, seed = NULL, ...)
                   },
                   if(!is.null(object$family$simulate))
                       object$family$simulate(object, nsim)
-                  else stop("family '", fam, "' not implemented"))
+                  else stop(gettextf("family '%s' not implemented", fam),
+                            domain = NA)
+                  )
 
     if(!is.list(val)) {
         dim(val) <- c(n, nsim)
@@ -525,11 +517,17 @@ model.frame.lm <- function(formula, ...)
     dots <- list(...)
     nargs <- dots[match(c("data", "na.action", "subset"), names(dots), 0)]
     if (length(nargs) || is.null(formula$model)) {
+        ## mimic lm(method = "model.frame")
         fcall <- formula$call
-        fcall$method <- "model.frame"
-        fcall[[1L]] <- as.name("lm")
+        m <- match(c("formula", "data", "subset", "weights", "na.action",
+                     "offset"), names(fcall), 0L)
+        fcall <- fcall[c(1L, m)]
+        fcall$drop.unused.levels <- TRUE
+        fcall[[1L]] <- as.name("model.frame")
+        fcall$xlev <- formula$xlevels
+        ## We want to copy over attributes here, especially predvars.
+        fcall$formula <- terms(formula)
         fcall[names(nargs)] <- nargs
-#	env <- environment(fcall$formula)  # always NULL
         env <- environment(formula$terms)
 	if (is.null(env)) env <- parent.frame()
         eval(fcall, env, parent.frame())
@@ -587,7 +585,7 @@ anova.lm <- function(object, ...)
     if(attr(object$terms,"intercept")) table <- table[-1, ]
     structure(table, heading = c("Analysis of Variance Table\n",
 		     paste("Response:", deparse(formula(object)[[2L]]))),
-	      class= c("anova", "data.frame"))# was "tabular"
+	      class = c("anova", "data.frame"))# was "tabular"
 }
 
 anova.lmlist <- function (object, ..., scale = 0, test = "F")
@@ -598,9 +596,9 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
     sameresp <- responses == responses[1L]
     if (!all(sameresp)) {
 	objects <- objects[sameresp]
-	warning("models with response ",
-                deparse(responses[!sameresp]),
-                " removed because response differs from ", "model 1")
+        warning(gettextf("models with response %s removed because response differs from model 1",
+                         sQuote(deparse(responses[!sameresp]))),
+                domain = NA)
     }
 
     ns <- sapply(objects, function(x) length(x$residuals))
@@ -628,7 +626,7 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
 
     title <- "Analysis of Variance Table\n"
     topnote <- paste("Model ", format(1L:nmodels),": ",
-		     variables, sep="", collapse="\n")
+		     variables, sep = "", collapse = "\n")
 
     ## calculate test statistic if needed
 
@@ -638,7 +636,7 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
 	table <- stat.anova(table = table, test = test,
 			    scale = scale,
                             df.scale = resdf[bigmodel],
-			    n = length(objects[bigmodel$residuals]))
+			    n = length(objects[[bigmodel]]$residuals))
     }
     structure(table, heading = c(title, topnote),
               class = c("anova", "data.frame"))
@@ -689,13 +687,12 @@ predict.lm <-
     interval <- match.arg(interval)
     if (interval == "prediction") {
         if (missing(newdata))
-            warning("Predictions on current data refer to _future_ responses\n")
-        if (missing(newdata) && missing(weights))
-        {
+            warning("predictions on current data refer to _future_ responses\n")
+        if (missing(newdata) && missing(weights)) {
             w <-  weights.default(object)
             if (!is.null(w)) {
                 weights <- w
-                warning("Assuming prediction variance inversely proportional to weights used for fitting\n")
+                warning("assuming prediction variance inversely proportional to weights used for fitting\n")
             }
         }
         if (!missing(newdata) && missing(weights) && !is.null(object$weights) && missing(pred.var))
@@ -819,7 +816,8 @@ predict.lm <-
     }
     if(se.fit || interval != "none") {
         se <- sqrt(ip)
-        if (type == "terms" && !is.null(terms)) se <- se[, terms, drop = FALSE]
+	if(type == "terms" && !is.null(terms) && !se.fit)
+	    se <- se[, terms, drop = FALSE]
     }
     if(missing(newdata) && !is.null(na.act <- object$na.action)) {
 	predictor <- napredict(na.act, predictor)
