@@ -1,6 +1,6 @@
-##--- S4 Methods (and Classes)
+####--- S4 Methods (and Classes)  --- see also ../src/library/methods/tests/
 options(useFancyQuotes=FALSE)
-library(methods)
+require(methods)
 ##too fragile: showMethods(where = "package:methods")
 
 ##-- S4 classes with S3 slots [moved from ./reg-tests-1.R]
@@ -211,7 +211,8 @@ f2 <- new("Foo", .Data = diag(2), name="Diag")# explicit .Data
 (m <- as(f, "matrix"))
 ## this has no longer (2.7.0) an S4 bit: set it explicitly just for testing:
 stopifnot(isS4(m. <- asS4(m)),
-          identical(m, f@.Data))
+          identical(m, f@.Data),
+	  .hasSlot(f, "name"))# failed in R <= 2.13.1
 show(m.)
 print(m.)
 ## fixed in 2.5.0 patched
@@ -310,13 +311,11 @@ as.raw(x_c1)
 # as.double, as.real use as.numeric for their methods to maintain equivalence
 setMethod("as.numeric", "c1", function(x, ...) 42+pi)
 identical(as.numeric(x_c1),as.double(x_c1))
-identical(as.numeric(x_c1),as.real(x_c1))
 
 
 setMethod(as.double, "c2", function(x, ...) x@.Data+pi)
 x_c2 <- new("c2", pi)
 identical(as.numeric(x_c2),as.double(x_c2))
-identical(as.numeric(x_c2),as.real(x_c2))
 
 ## '!' changed signature from 'e1' to 'x' in 2.6.0
 setClass("foo", "logical")
@@ -330,7 +329,7 @@ setMethod("as.vector", signature(x = "foo", mode = "missing"),
           function(x) unclass(x))
 ## whereas this fails in R versions earlier than 2.6.0:
 setMethod("as.vector", "foo", function(x) unclass(x))
-xx <- removeClass("foo")
+stopifnot(removeClass("foo"))
 
 ## stats4::AIC in R < 2.7.0 used to clobber stats::AIC
 pfit <- function(data) {
@@ -471,7 +470,7 @@ setMethod("Summary", "L",
 setMethod("[", signature(x="L", i="ANY", j="missing",drop="missing"),
           function(x,i,j,drop) new(class(x), x@.Data[i]))
 ## defining S4 methods for sort() has no effect on calls to
-## sort() from functions in a name space; e.g., median.default.
+## sort() from functions in a namespace; e.g., median.default.
 ## but setting an xtfrm() method works.
 setMethod("xtfrm", "L", function(x) xtfrm(unlist(x@.Data)))
 ## median is documented to use mean(), so we need an S3 mean method:
@@ -529,3 +528,84 @@ x[1, drop=FALSE]
 y[1, drop=FALSE]
 ## the last gave TRUE on C1-level in R 2.10.x;
 ## the value of drop was wrongly taken from the default.
+
+## All slot names -- but "class" -- should work now
+problNames <- c("names", "dimnames", "row.names",
+                "class", "comment", "dim", "tsp")
+myTry <- function(expr, ...) tryCatch(expr, error = function(e) e)
+tstSlotname <- function(nm) {
+    r <- myTry(setClass("foo", representation =
+                        structure(list("character"), .Names = nm)))
+    if(is(r, "error")) return(r$message)
+    ## else
+    ch <- LETTERS[1:5]
+    ## instead of  new("foo", <...> = ch):
+    x <- myTry(do.call(new, structure(list("foo", ch), .Names=c("", nm))))
+    if(is(x, "error")) return(x$message)
+    y <- myTry(new("foo"));		 if(is(y, "error")) return(y$message)
+    r <- myTry(capture.output(show(x))); if(is(r, "error")) return(r$message)
+    r <- myTry(capture.output(show(y))); if(is(r, "error")) return(r$message)
+    ## else
+    slot(y, nm) <- slot(x, nm)
+    stopifnot(validObject(x), identical(x,y), identical(slot(x, nm), ch))
+    return(TRUE)
+}
+R <- sapply(problNames, tstSlotname, simplify = FALSE)
+str(R) # just so ...
+stopifnot(is.character(R[["class"]]),
+          sapply(R[names(R) != "class"], isTRUE))
+## only "class" (and ".Data", ...) is reserved as slot name
+
+## implicit generics ..
+setMethod("sample", "C2",
+          function(x, size, replace=FALSE, prob=NULL) {"sample.C2"})
+stopifnot(is(sample,"standardGeneric"),
+	  ## the signature must come from the implicit generic:
+	  identical(sample@signature, c("x", "size")),
+	  identical(packageSlot(sample), "base"),
+	  ## default method must still work:
+	  identical({set.seed(3); sample(3)}, 1:3))
+## failed in R 2.11.0
+
+## Still, signature is taken from "def"inition, if one is provided:
+## (For test, qqplot must be a "simple" function:)
+stopifnot(is.function(qqplot) && identical(class(qqplot), "function"))
+setGeneric("qqplot", function(x, y, ...) standardGeneric("qqplot"))
+stopifnot(is(qqplot, "standardGeneric"),
+	  identical(qqplot@signature, c("x","y")))
+## failed for a day ~ 2005-05-26, for R-devel only
+
+
+##  'L$A@x <- ..'
+setClass("foo", representation(x = "numeric"))
+f <- new("foo", x = pi*1:2)
+L <- list()
+L$A <- f
+L$A@x[] <- 7
+if( identical(f, L$A) )
+    stop("Oops! f is identical to L$A, even though not touched!")
+## did not duplicate in 2.0.0 <= Rversion <= 2.11.1
+
+
+## prototypes for virtual classes:  NULL if legal, otherwise 1st member
+## OptionalPosixct above includes NULL
+stopifnot(is.null(getClass("OptionalPOSIXct")@prototype))
+## "IntOrChar" had invalid NULL prototype < 2.15.0
+setClassUnion("IntOrChar", c("integer", "character"))
+stopifnot(is.integer(getClass("IntOrChar")@prototype))
+## produced an error < 2.15.0
+stopifnot(identical(isGeneric("&&"), FALSE))
+
+
+## mapply() on S4 objects with a "non-primitive" length() method
+setClass("A", representation(aa="integer"))
+aa <- 11:16
+a <- new("A", aa=aa)
+setMethod(length, "A", function(x) length(x@aa))
+setMethod(`[[`,   "A", function(x, i, j, ...) x@aa[[i]])
+stopifnot(length(a) == 6, identical(a[[5]], aa[[5]]),
+	  identical(mapply(`*`, aa, rep(1:3, 2)),
+		    mapply(`*`, a,  rep(1:3, 2))))
+## Up to R 2.15.2, internally 'a' is treated as if it was of length 1
+## because internal dispatch did not work for length().
+
